@@ -36,10 +36,6 @@ import (
 	"github.com/goplus/gox"
 )
 
-const (
-	gopPrefix = "Gop_"
-)
-
 const (
 	DbgFlagLoad = 1 << iota
 	DbgFlagLookup
@@ -507,6 +503,7 @@ type pkgCtx struct {
 	*gmxSettings
 	syms  map[string]loader
 	inits []func()
+	tylds []*typeLoader
 	errs  []error
 }
 
@@ -628,7 +625,6 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 		LoadNamed:       ctx.loadNamed,
 		HandleErr:       ctx.handleErr,
 		NodeInterpreter: interp,
-		Prefix:          gopPrefix,
 		ParseFile:       nil, // TODO
 		NewBuiltin:      newBuiltinDefault,
 	}
@@ -645,11 +641,11 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 	for _, f := range pkg.Files {
 		if f.FileType == ast.FileTypeGmx {
 			loadFile(ctx, f)
-			if o := p.Types.Scope().Lookup(ctx.Class); o != nil && hasMethod(o, "main") {
+			if o := p.Types.Scope().Lookup(ctx.Class); o != nil && hasMethod(o, "Main") {
 				// new(Game).main()
 				p.NewFunc(nil, "main", nil, nil, false).BodyStart(p).
 					Val(p.Builtin().Ref("new")).Val(o).Call(1).
-					MemberVal("main").Call(0).EndStmt().
+					MemberVal("Main").Call(0).EndStmt().
 					End()
 			}
 			break
@@ -660,6 +656,9 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 			loadFile(ctx, f)
 		}
 	}
+	for _, ld := range ctx.tylds {
+		ld.load()
+	}
 	for _, load := range ctx.inits {
 		load()
 	}
@@ -765,6 +764,7 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 				}
 				decl.InitType(p, types.NewStruct(flds, nil))
 			}
+			parent.tylds = append(parent.tylds, ld)
 		}
 		ctx.classRecv = &ast.FieldList{List: []*ast.Field{{
 			Names: []*ast.Ident{
@@ -780,9 +780,14 @@ func preloadFile(p *gox.Package, parent *pkgCtx, file string, f *ast.File, targe
 		case *ast.FuncDecl:
 			if ctx.classRecv != nil { // in class file (.spx/.gmx)
 				if d.Recv == nil {
+					name := d.Name.Name
 					d.Recv = ctx.classRecv
-					ctx.addParams(d.Name.Name, d.Type)
-					ctx.toTitle(d.Name)
+					ctx.addParams(name, d.Type)
+					if f.NoEntrypoint && (name == "main") {
+						d.Name.Name = "Main"
+					} else { // onMsg => OnMsg, etc
+						ctx.toTitle(d.Name)
+					}
 				}
 			}
 			if d.Recv == nil {
