@@ -127,6 +127,29 @@ func init() {
 `)
 }
 
+func TestRedefineBuiltin(t *testing.T) {
+	gopClTest(t, `
+func main() {
+	const a = append + len
+}
+
+const (
+	append = iota
+	len
+)
+`, `package main
+
+func main() {
+	const a = append + len
+}
+
+const (
+	append = iota
+	len
+)
+`)
+}
+
 func TestTypeConvIssue804(t *testing.T) {
 	gopClTest(t, `
 c := make(chan int)
@@ -559,7 +582,7 @@ func main() {
 	fmt.Println(a.(*A))
 }
 
-func get()AA{
+func get() AA {
 	var a AA
 	return a
 }
@@ -1033,27 +1056,27 @@ func add(x, y string) (int, error) {
 import strconv "strconv"
 
 func add(x string, y string) (int, error) {
-	var _autoGop_1 int
+	var _autoGo_1 int
 	{
 		var _gop_err error
-		_autoGop_1, _gop_err = strconv.Atoi(x)
+		_autoGo_1, _gop_err = strconv.Atoi(x)
 		if _gop_err != nil {
 			return 0, _gop_err
 		}
-		goto _autoGop_2
-	_autoGop_2:
+		goto _autoGo_2
+	_autoGo_2:
 	}
-	var _autoGop_3 int
+	var _autoGo_3 int
 	{
 		var _gop_err error
-		_autoGop_3, _gop_err = strconv.Atoi(y)
+		_autoGo_3, _gop_err = strconv.Atoi(y)
 		if _gop_err != nil {
 			return 0, _gop_err
 		}
-		goto _autoGop_4
-	_autoGop_4:
+		goto _autoGo_4
+	_autoGo_4:
 	}
-	return _autoGop_1 + _autoGop_3, nil
+	return _autoGo_1 + _autoGo_3, nil
 }
 `)
 }
@@ -2632,6 +2655,29 @@ func main() {
 `)
 }
 
+func TestLambdaExpr2(t *testing.T) {
+	gopClTest(t, `
+func Do(func()) {
+	// ...
+}
+
+Do => {
+	println "Hi"
+}
+`, `package main
+
+import fmt "fmt"
+
+func Do(func()) {
+}
+func main() {
+	Do(func() {
+		fmt.Println("Hi")
+	})
+}
+`)
+}
+
 func TestUnnamedMainFunc(t *testing.T) {
 	gopClTest(t, `i := 1`, `package main
 
